function return_data = plate_cavity_rad_tau(f,plate_values,cavity_values,plate_clf_tot)
% plate_cavity_rad_tau(f,plate_values,cavity_values,plate_clf_tot)
%
% [Usage]:
%   radeff_tau = plate_cavity_rad_tau(f,plate_values,cavity_values,plate_clf_tot)
%
% [Inputs]:
%   f (double array) - frequency vector
%   plate_values (structure array)
%       - fields:
%           c (structure array)
%               -fields:
%                   b - bending wavespeed in the plate.
%                   l - longitudinal wavespeed in the plate.
%                   s - shear wavespeed in the plate.
%           thickness - plate thickness [m].
%           rho - plate density.
%           dim - double array with two values:
%                 first = plate length.
%                 second = plate width.
%   cavity values (structure array)
%       - fields:
%           c - wave speed in the fluid.
%           rho - density of the fluid.
%   plate_clf_tot - total loss factor for the plate.
%
% [Output]:
%   radeff_tau (structure array)
%       -fields:
%           rad(ii) - struture array over ii cavities connected to the
%                     plate
%               - fields:
%                   rad (double array) - radiation efficiency over
%                                        frequency
%           tau(ii) - struture array over ii cavities connected to the
%                     plate
%               -fields:
%                   tau (double array) - transmission coefficient over
%                                        frequency for low frequency
%                                        transmission between the two
%                                        cavities (if there are two)
%

%
    numcav = length(cavity_values);
    % get plate values.
    c = plate_values.c;
    h = plate_values.thickness;
    rho = plate_values.rho*h;
    l_1 = max(plate_values.dim);
    l_2 = min(plate_values.dim);
    % predefining variables to prevent realocating memory in for loops.
    c0(1:numcav) = 0;
    rho0(1:numcav) = 0;
    k0(1:numcav) = struct('k0',zeros(1,length(f)));
    tau(1:numcav) = struct('tau',zeros(1,length(f)));
    tau_all(1:numcav) = struct('tau',zeros(1,length(f)));
    rad_(1:numcav) = struct('rad_1',zeros(1,length(f)),'rad_2',zeros(1,length(f)),'rad_3',zeros(1,length(f)));
    rad(1:numcav) = struct('rad',zeros(1,length(f)));
    lambda(1:numcav) = struct('l',zeros(1,length(f)),'L',zeros(1,length(f)));
    fc(1:numcav) = NaN;
    f11(1:numcav) = NaN;
    delta(1:numcav) = struct('d1',zeros(1,length(f)));
    % get cavity values.
    for cavs = 1:length(cavity_values)
        c0(cavs) = cavity_values.c;
        k0(cavs).k0 = (2*pi*f)/c0(cavs);
        rho0(cavs) = cavity_values.rho;
    end
    % calculate radiation efficiency according to standard 12354-1.
    for cons = 1:length(cavity_values)
        fc(cons) = c0(cons)^2./(1.8*c.l(1)*h);
        rad_(cons).rad_1 = 1./sqrt(1-fc(cons)./f);
        rad_(cons).rad_2 = 4*l_1*l_2*(f/c0(cons)).^2;
        rad_(cons).rad_3 = sqrt(2*pi*f*(l_1+l_2)/(16*c0(cons)));
        f11(cons) = c0(cons)^2*(1/l_1^2+1/l_2^2)/(4*fc(cons));
        lambda(cons).l = sqrt(f/fc(cons));
        delta(cons).d1 = ((1-lambda(cons).l.^2).*log((1+lambda(cons).l)./(1-lambda(cons).l))+2*lambda(cons).l)./(4*pi^2*(1-lambda(cons).l.^2).^(1.5));
        delta(cons).d2 = (8*c0(cons)^2*(1-2*lambda(cons).l.^2)./(fc(cons)^2*pi^4*l_1*l_2*lambda(cons).l.*sqrt(1-lambda(cons).l.^2)));
        delta(cons).d2 = delta(cons).d2.*(f<=fc(cons)/2);
    end
    for cons = 1:numcav
        if f11(cons) < fc(cons)/2
            pos1 = find(f>=fc(cons));
            rad(cons).rad(pos1) = rad_(cons).rad_1(pos1);
            pos2 = find(f<fc(cons));
            rad(cons).rad(pos2) = 2*(l_1+l_2)*c0(cons)*delta(cons).d1(pos2)/(l_1*l_2*fc(cons))+delta(cons).d2(pos2);
            pos3 = find(f<f11(cons) & f<fc(cons)/2 & rad(cons).rad>rad_(cons).rad_2);
            rad(cons).rad(pos3) = rad_(cons).rad_2(pos3);
            rad(cons).rad = min(rad(cons).rad,2);
        else
            pos1 = find(f<fc(cons) & rad_(cons).rad_2<rad_(cons).rad_3);
            rad(cons).rad(pos1) = rad_(cons).rad_2(pos1);
            pos2 = find(f>fc(cons) & rad_(cons).rad_1<rad_(cons).rad_3);
            rad(cons).rad(pos2) = rad_(cons).rad_1(pos2);
            pos3 = find(rad(cons).rad == 0);
            rad(cons).rad(pos3) = rad_(cons).rad_3(pos3);
            rad(cons).rad = min(rad(cons).rad,2);
        end
        % calculate the transmission coefficient between the cavities.
        lambda(cons).L = -0.964-(0.5+l_2/(pi*l_1))*log(l_2/l_1)+5*l_2/(2*pi*l_1)-1./(4*pi*l_1*l_2*k0(cavs).k0.^2);
        rad_(cons).f = min(0.5*(log(k0(cons).k0*sqrt(l_1*l_2))-lambda(cons).L),2);
        tau_all(cons).tau = (2*rho0(cons)*c0(cons)./(2*pi*f*rho)).^2.*(2*rad_(cons).f+(l_1+l_2)^2/(l_1^2+l_2^2).*sqrt(fc(cons)./f).*rad(cons).rad.^2./plate_clf_tot);
        tau(cons).tau(f<fc(cons)) = tau_all(cons).tau(f<fc(cons));
    end
    % return radiation efficiency and transmission coefficients.
    return_data.rad = rad;
    return_data.tau = tau;