function return_data = tau_plate_plate(f,connection_values,subsystem_values)
% tau_plate_plate(f,connection_values,subsystem_values) calculates the
% transmission coefficients between an arbitrary number of plates connected
% over a line junction. Any connection angles between the plates can be
% used.
% The transmission coefficient for an oblique incident angle will be
% calculated so a number of incident angles is treated and then the
% integral over those angles is calculated to get the transmission
% coefficient for the diffuse sound field in the plates.
%
% [Usage]:
%   tau = tau_plate_plate(f,connection_values,subsystem_values)
%
% [Input]:
%   f (double array) - frequency vector
%   connection_values (structure array)
%       - fields:
%           num_angles - number of incident angles treated.
%           angles (double array) - connection angles. angle to the first
%                                   plate in the connection (see JC Figure 4.1).
%   subsystem_values (structure array)
%       - fields:
%           thickness (double array) - thickness of each plate [m].
%           E (double array) - youngs modulus for each plate.
%           poi (double array) - Poissions ratio for each plate.
%           rho (double array) -  material density of each plate.
%           D (double array) - bending stiffness of each plate.
%           k (structure array)
%               -fields:
%                   b (double array) - bending wave number over frequency.
%                   l (double array) - longitudinal wave number over
%                                      frequency.
%                   s (double array) - shear wave number over frequency.
%
% [Output]:
%   tau(from,to,wavetype) (structure array) - the transmission coefficients
%                                             where 'from' and 'to'
%                                             represents between which
%                                             subsystems and wavetype the
%                                             type of incident wave it is.
%                                             wavetype = 1 means a bending,
%                                             2 a longitudinal and 3 a
%                                             shear incident wave.
%       fields:
%           b (double array) - represents the transmission coefficient to a
%                              bending wave.
%           l (double array) - represents the transmission coefficient to a
%                              longitudinal wave.
%           s (double array) - represents the transmission coefficient to a
%                              shear wave.
%

% 110411 PA tau_plate_plate.m 
%           Changed so that the angle step (ang_dif) corresponds to (ang_center),
%           both variables are used in the final intergation of the transmission coefficient 
%           over the angle of incident wave to get the diffuse field tranmission coefficient (JC 4.45),
%           (ang-dif) does now have a values that is unique for each incident angle considered
%           (befor the (ang_dif) variable had a constant value for all angles which was wrong)
% 110428 PA tau_plate_plate.m
%           Trapezoidal rule instead of midpoint/rectangular rule when calculating the diffuse
%           field tranmission coefficient by taking the intrgral of transmission coefficients
%           over angles of incident waves between 0 and 90 (JC 4.45). 
%           Removed variables: 'ang_edge' (limits of bins) and 'ang_center' (center of bins) 
%           New variable: 'ang_ev' angles of incident waves for which the integral is evaluated
%               using the trapezoidal rule
% 110428 PA tau_plate_plate.m 
%           Added more comments
% 110428 PA tau_plate_plate.m
%           Added references to equations in Johansson/Comnell thesis (JC x.xx) and Langley/Heron paper (LH xx)
% 110831 PA tau_plate_plate.m
%           Corrected bug: Elements of sum(R_j*K*R_j^T) (aL11sum, aL12sum, ...) was only calculated 
%           for the case with the wave incident on the plate with index 1 at the junction. Now it is 
%           calculated uniquely for each incident wave on each plate. In specific:
%               * Removed 'if from==1' at row 286 (row in old code)
%               * Made one 'for form='-loop for setting up the sum(R_j*K*R_j^T) matrix and 
%                   solving the equation system (they were separated before)
%               * Clear the sum(R_j*K*R_j^T)-valiables (aL11sum, aL12sum, ...) at the start of the 'for form='-loop, 

% Predefine variables to prevent realocation of memory in for loops.
numss = length(subsystem_values);
h(1:numss) = NaN;
E(1:numss) = struct('y',[]);
poi(1:numss) = NaN;
rho(1:numss) = NaN;
D(1:numss) = NaN;
k(1:numss) = struct('b',[],'l',[],'s',[]);
ju(1:numss) = struct('kb',[],'kl',[],'ks',[],'myb',[],'myl',[],'mys',[]);
angtr(1:numss,1:numss) = struct('waveB',[],'waveL',[],'waveS',[]);
R(1:numss) = struct('R22',[],'R23',[],'R32',[],'R33',[],'R42',[],'R43',[]);
K(1:numss,1:numss) = struct('K11',[],'K12',[],'K21',[],'K22',[],'K33',[],'K34',[],'K43',[],'K44',[],'Ka11',[],'Ka12',[],'Ka21',[],'Ka22',[],'Kb21',[],'Kb22',[]);
my(1:numss,1:numss) = struct('l_1',[],'l_2',[],'s_1',[],'s_2',[],'b_1',[],'b_2',[],'b_3',[],'b_4',[],'l',[],'s',[],'b1',[],'b2',[],'bp',[]);
bp(1:numss) = struct('p_1',[],'p_2',[],'p_3',[],'p_4',[]);
F(1:numss) = struct('p_1',[],'p_2',[],'p_3',[],'p_4',[]);
fm(1:numss) = struct('fm_1',[],'fm_2',[],'fm_3',[],'fm_4',[]);
b(1:numss,1:numss) = struct('b1',[],'b2',[],'b3',[],'b4',[]);
a(1:numss,1:numss) = struct('aL11',[],'aL12',[],'aL13',[],'aL14',[],'aL21',[],'aL22',[],'aL23',[],'aL24',[],'aL31',[],'aL32',[],'aL33',[],'aL34',[],'aL41',[],'aL42',[],'aL43',[],'aL44',[],'aR1',[],'aR2',[],'aR3',[],'aR4',[],'a1',[],'a2',[],'a3',[],'a4',[]);
dis(1:numss,1:numss) = struct('dis1',[],'dis2',[],'dis3',[],'dis4',[],'dismp1',[],'dismp2',[],'dismp3',[],'dismp4',[]);
P(1:numss,1:numss) = struct('Pb1',[],'Pb2',[],'Pl',[],'Ps',[]);
Pin(1:numss) = struct('Pin',[]);
ang(1:numss) = struct('beam',[],'wave',[]);
y(1:numss) = NaN;
z(1:numss) = NaN;
alfa(1:numss) = struct('b',[],'l',[],'s',[]);
tau(1:numss,1:numss) = struct('b',[],'l',[],'s',[]);
tau_ang(1:numss,1:numss) = struct('b1',[],'b2',[],'l',[],'s',[]);

% Create the incident angle vector used when integrating the tranmission coefficients to get the
%   diffuse field transmission coefficient. The angles are "cos spaced" to have more angles closer
%   to where the incident wave is perpendicular to the junction.
ang_ev=acosd(linspace(1,0,connection_values.num_angles+1)); % Incident angles for which the integral is evaluated 0 ... 90 degree
ang_dif=pi/180*diff(ang_ev); % Corresponding angle step in radians
for ii = 1:numss
    ang(ii).beam = connection_values.angles(ii); % connection angle relative the "first" plate in the junction.
    y(ii) = 0; % Coordinates to an arbitrary point in the cross-section of the beam, (not implemented)
    z(ii) = 0; % Coordinates to an arbitrary point in the cross-section of the beam, (not implemented)
    ang(ii).wave = ang_ev; % incident wave vector
end
% y0 = 0; % Y-coordinate of the centroid, (not implemented)
% z0 = 0; % Z-coordinate of the centroid, (not implemented)
w = 2*pi*f; % angular frequency
numang = length(ang(1).wave); % number of incident angles.
numf = length(f); % number of frequencies.
% get subsystem values.
for subs = 1:numss
    h(subs) = subsystem_values(subs).thickness;
    E(subs).y = subsystem_values(subs).E;
    poi(subs) = subsystem_values(subs).poi;
    rho(subs) = subsystem_values(subs).rho.*subsystem_values(subs).thickness;
    D(subs) = subsystem_values(subs).D;
    k(subs) = subsystem_values(subs).k;
end
% calculate the angle dependent elements in the R-matrix, JC (4.23), LH (15) 
for subs = 1:numss
    R(subs).R22 = ones(numang,numf)*cosd(ang(subs).beam);
    R(subs).R23 = ones(numang,numf)*(-sind(ang(subs).beam));
    R(subs).R32 = ones(numang,numf)*sind(ang(subs).beam);
    R(subs).R33 = ones(numang,numf)*cosd(ang(subs).beam);
    R(subs).R42 = ones(numang,numf)*(y(subs)*sind(ang(subs).beam)-z(subs)*cosd(ang(subs).beam));
    R(subs).R43 = ones(numang,numf)*(y(subs)*cosd(ang(subs).beam)+z(subs)*sind(ang(subs).beam));
end
% calculate the wave number at the junction.
for from = 1:numss
    ju(from).kb = cosd(ang(from).wave)'*k(from).b; % JC (4.29), LH below (35)
    ju(from).kl = cosd(ang(from).wave)'*k(from).l; % JC (4.33), LH below (39)
    ju(from).ks = cosd(ang(from).wave)'*k(from).s; % JC (4.37), LH below (43)
    ju(from).myb = 1i*(sind(ang(from).wave.')*ones(1,numf)).*(ones(numang,1)*k(from).b); % JC (4.30), LH below (35)
    ju(from).myl = 1i*(sind(ang(from).wave.')*ones(1,numf)).*(ones(numang,1)*k(from).l); % JC (4.34), LH below (39)
    ju(from).mys = 1i*(sind(ang(from).wave.')*ones(1,numf)).*(ones(numang,1)*k(from).s); % JC (4.38), LH below (43)
 end
% setting the displacement of the incident wave to an arbitrary size, 1E-10 as a reference
for subs = 1:numss
    alfa(subs).b(1:numang,1:numf) = 1E-10;
    alfa(subs).l(1:numang,1:numf) = 1E-10;
    alfa(subs).s(1:numang,1:numf) = 1E-10;
end

% loop calculations for each wave type considered. 1-bending,
% 2-longitudinal, 3-shear waves.
for wavetype = 1:3
    % use the corrent wave number on the junction for the wavetype considered.
    switch wavetype
        case 1 % bending wave
            for from = 1:numss
                ju(from).k = ju(from).kb;
            end
        case 2 % longitudinal wave
            for from = 1:numss
                ju(from).k = ju(from).kl;
            end
        case 3 % shear wave.
            for subs = 1:numss
                ju(subs).k = ju(subs).ks;
            end
    end
    
    % calculate 'my' the wave number in y-direction in all plates for all wave types and the
    % corresponding the transmitted wave angle and the stiffness matrix for each subsystem combination.
    for from = 1:numss
        % predefining some matrixes to prevent realocating memory
        % Elements of sum(R_j*K*R_j^T), JC (4.24) LH SIDE
        aL11sum = zeros(numang,numf);
        aL12sum = zeros(numang,numf);
        aL13sum = zeros(numang,numf);
        aL14sum = zeros(numang,numf);
        aL21sum = zeros(numang,numf);
        aL22sum = zeros(numang,numf);
        aL23sum = zeros(numang,numf);
        aL24sum = zeros(numang,numf);
        aL31sum = zeros(numang,numf);
        aL32sum = zeros(numang,numf);
        aL33sum = zeros(numang,numf);
        aL34sum = zeros(numang,numf);
        aL41sum = zeros(numang,numf);
        aL42sum = zeros(numang,numf);
        aL43sum = zeros(numang,numf);
        aL44sum = zeros(numang,numf);
        
        for subs = 1:numss
            % Calculate the valid possible roots for 'my' for each wavetype
            % Calculate the wave number in y-direction of the trasmitted waves
            my(from,subs).l = -sqrt(ju(from).k.^2-ones(numang,1)*k(subs).l.^2); % JC (4.5), LH (22)
            my(from,subs).s = -sqrt(ju(from).k.^2-ones(numang,1)*k(subs).s.^2); % JC (4.6), LH (23)
            my(from,subs).b1 = -sqrt(ju(from).k.^2+ones(numang,1)*k(subs).b.^2); % JC (4.4), LH (17)
            my(from,subs).b2 = -sqrt(ju(from).k.^2-ones(numang,1)*k(subs).b.^2); % JC (4.4), LH (17)
            % Calculate the angle of the transmitted waves
            angtr(from,subs).waveB = acosd(ju(from).k./(ones(numang,1)*k(subs).b)); % JC (4.43), LH p 247 row end-4
            angtr(from,subs).waveL = acosd(ju(from).k./(ones(numang,1)*k(subs).l)); % JC (4.43), LH p 247 row end-4
            angtr(from,subs).waveS = acosd(ju(from).k./(ones(numang,1)*k(subs).s)); % JC (4.43), LH p 247 row end-4
            % 
            % Matrix for relation between edge displacements and tractions of each plate
            % JC (4.9-4.16), LH (28,21) where K12, K21 and K43 has been corrected
            %  (checked 110428 by PA)
            K(from,subs).K11 = -(my(from,subs).s.^2-ju(from).k.^2).*my(from,subs).l/(2*(1+poi(subs)));
            K(from,subs).K11 = K(from,subs).K11.*(E(subs).y*h(subs)./(ju(from).k.^2-my(from,subs).s.*my(from,subs).l));
            K(from,subs).K12 = -1i*ju(from).k.*(my(from,subs).s.^2+ju(from).k.^2-2*my(from,subs).l.*my(from,subs).s)/(2*(1+poi(subs)));
            K(from,subs).K12 = K(from,subs).K12.*(E(subs).y*h(subs)./(ju(from).k.^2-my(from,subs).s.*my(from,subs).l));
            K(from,subs).K21 = 1i*(my(from,subs).l.^2-poi(subs)*ju(from).k.^2).*ju(from).k/(1-poi(subs)^2)-1i*my(from,subs).s.*my(from,subs).l.*ju(from).k/(1+poi(subs));
            K(from,subs).K21 = K(from,subs).K21.*(E(subs).y*h(subs)./(ju(from).k.^2-my(from,subs).s.*my(from,subs).l));
            K(from,subs).K22 = (poi(subs)*ju(from).k.^2-my(from,subs).l.^2).*my(from,subs).s/(1-poi(subs)^2)+ju(from).k.^2.*my(from,subs).s/(1+poi(subs));
            K(from,subs).K22 = K(from,subs).K22.*(E(subs).y*h(subs)./(ju(from).k.^2-my(from,subs).s.*my(from,subs).l));
            K(from,subs).K33 = my(from,subs).b1.^3.*my(from,subs).b2-my(from,subs).b2.^3.*my(from,subs).b1;
            K(from,subs).K33 = K(from,subs).K33.*(D(subs)./(my(from,subs).b1-my(from,subs).b2));
            K(from,subs).K34 = my(from,subs).b2.^3-my(from,subs).b1.^3+(2-poi(subs))*(my(from,subs).b1-my(from,subs).b2).*ju(from).k.^2;
            K(from,subs).K34 = K(from,subs).K34.*(D(subs)./(my(from,subs).b1-my(from,subs).b2));
            K(from,subs).K43 = my(from,subs).b1.*my(from,subs).b2.^2-my(from,subs).b2.*my(from,subs).b1.^2+poi(subs)*(my(from,subs).b2-my(from,subs).b1).*ju(from).k.^2;
            K(from,subs).K43 = K(from,subs).K43.*(D(subs)./(my(from,subs).b1-my(from,subs).b2));
            K(from,subs).K44 = my(from,subs).b1.^2-my(from,subs).b2.^2;
            K(from,subs).K44 = K(from,subs).K44.*(D(subs)./(my(from,subs).b1-my(from,subs).b2));
            % Matrix for relation between wave amplitudes and edge displacements 
            % JC (4.39), LH (27,20)
            K(from,subs).Ka11 = ju(from).k;
            K(from,subs).Ka12 = 1i*my(from,subs).s;
            K(from,subs).Ka21 = 1i*my(from,subs).l;
            K(from,subs).Ka22 = -ju(from).k;
            K(from,subs).Kb21 = my(from,subs).b1;
            K(from,subs).Kb22 = my(from,subs).b2;
        end
        % calculate the edge displacements and edge tractions for each wavetype.
        switch wavetype
            case 1 % bending wave
                ju(from).myp = ju(from).myb; % only a local variable used here
                % JC (4.28) edge displacements due to incident bending wave
                bp(from).p_1 = 0;
                bp(from).p_2 = 0;
                bp(from).p_3 = alfa(from).b;
                bp(from).p_4 = alfa(from).b.*ju(from).myp;
                % JC (4.27) edge tractions due to incident bending wave
                F(from).p_1 = 0;
                F(from).p_2 = 0;
                F(from).p_3 = -alfa(from).b*D(from).*(ju(from).myp.^3-(2-poi(from))*ju(from).k.^2.*ju(from).myp);
                F(from).p_4 = alfa(from).b*D(from).*(ju(from).myp.^2-poi(from)*ju(from).k.^2);
            case 2 % longitudinal wave
                %
                ju(from).myp = ju(from).myl; % only a local variable used here
                % JC (4.32) edge displacements due to incident longitudinal wave
                bp(from).p_1 = alfa(from).l.*ju(from).k;
                bp(from).p_2 = alfa(from).l.*(1i*ju(from).myp);
                bp(from).p_3 = 0;
                bp(from).p_4 = 0;
                % JC (4.31) edge tractions due to incident longitudinal wave
                F(from).p_1 = alfa(from).l*E(from).y*h(from).*ju(from).k.*ju(from).myp/(1+poi(from));
                F(from).p_2 = alfa(from).l*1i*E(from).y*h(from).*( ju(from).myp.^2-poi(from)*ju(from).k.^2)/(1-poi(from)^2);
                F(from).p_3 = 0;
                F(from).p_4 = 0;
            case 3 % shear wave
                ju(from).myp = ju(from).mys; % only a local variable used here
                % JC (4.36) edge displacements due to incident shear wave
                bp(from).p_1 = 1i*alfa(from).s.*ju(from).myp;
                bp(from).p_2 = -alfa(from).s.*ju(from).k;
                bp(from).p_3 = 0;
                bp(from).p_4 = 0;
                % JC (4.35) edge tractions due to incident shear wave
                F(from).p_1 = alfa(from).s*1i*E(from).y*h(from).*(ju(from).myp.^2+ju(from).k.^2)/(2*(1+poi(from)));
                F(from).p_2 = -alfa(from).s*E(from).y*h(from).*ju(from).k.*ju(from).myp/(1+poi(from));
                F(from).p_3 = 0;
                F(from).p_4 = 0;
        end
        % calculating the dynamic loading vector acting on the
        % semi-infinite plate carrying the incident wave.
        % JC (4.26)
        fm(from).fm_1 = K(from,from).K11.*bp(from).p_1 - F(from).p_1 + K(from,from).K12.*bp(from).p_2;
        fm(from).fm_2 = K(from,from).K21.*bp(from).p_1 - F(from).p_2 + K(from,from).K22.*bp(from).p_2;
        fm(from).fm_3 = K(from,from).K33.*bp(from).p_3 - F(from).p_3 + K(from,from).K34.*bp(from).p_4;
        fm(from).fm_4 = K(from,from).K43.*bp(from).p_3 - F(from).p_4 + K(from,from).K44.*bp(from).p_4;
        % calculating the product of the rectangular matrix R and the
        % dynamic loading vector calculated above for the plate carying the
        % incident wave.
        % JC (4.24) RH side
        a(from).aR1 = fm(from).fm_1;
        a(from).aR2 = fm(from).fm_2.*R(from).R22 + fm(from).fm_3.*R(from).R23;
        a(from).aR3 = fm(from).fm_2.*R(from).R32 + fm(from).fm_3.*R(from).R33;
        a(from).aR4 = fm(from).fm_4 + fm(from).fm_2.*R(from).R42 + fm(from).fm_3.*R(from).R43;
        % calculating the sum of each RKR' equation for each subsystem.
        % JC (4.24) LH SIDE
        for subs = 1:numss
            % Elements of sum(R_j*K*R_j^T), JC (4.24) LH SIDE (checked 110413 by PA)
            aL11sum = aL11sum + K(from,subs).K11;
            aL12sum = aL12sum + K(from,subs).K12.*R(subs).R22;
            aL13sum = aL13sum + K(from,subs).K12.*R(subs).R32;
            aL14sum = aL14sum + K(from,subs).K12.*R(subs).R42;
            aL21sum = aL21sum + K(from,subs).K21.*R(subs).R22 ;
            aL22sum = aL22sum + K(from,subs).K22.*R(subs).R22.^2+K(from,subs).K33.*R(subs).R23.^2;
            aL23sum = aL23sum + K(from,subs).K22.*R(subs).R22.*R(subs).R32+K(from,subs).K33.*R(subs).R23.*R(subs).R33;
            aL24sum = aL24sum + K(from,subs).K34.*R(subs).R23+K(from,subs).K22.*R(subs).R22.*R(subs).R42+K(from,subs).K33.*R(subs).R23.*R(subs).R43;
            aL31sum = aL31sum + K(from,subs).K21.*R(subs).R32;
            aL32sum = aL32sum + K(from,subs).K22.*R(subs).R22.*R(subs).R32+K(from,subs).K33.*R(subs).R23.*R(subs).R33;
            aL33sum = aL33sum + K(from,subs).K22.*R(subs).R32.^2+K(from,subs).K33.*R(subs).R33.^2;
            aL34sum = aL34sum + K(from,subs).K34.*R(subs).R33+K(from,subs).K22.*R(subs).R32.*R(subs).R42+K(from,subs).K33.*R(subs).R33.*R(subs).R43;
            aL41sum = aL41sum + K(from,subs).K21.*R(subs).R42;
            aL42sum = aL42sum + R(subs).R23.*(K(from,subs).K43+K(from,subs).K33.*R(subs).R43)+K(from,subs).K22.*R(subs).R22.*R(subs).R42;
            aL43sum = aL43sum + R(subs).R33.*(K(from,subs).K43+K(from,subs).K33.*R(subs).R43)+K(from,subs).K22.*R(subs).R32.*R(subs).R42;
            aL44sum = aL44sum + K(from,subs).K22.*R(subs).R42.^2+K(from,subs).K44+K(from,subs).K34.*R(subs).R43+R(subs).R43.*(K(from,subs).K43+K(from,subs).K33.*R(subs).R43);
        end
        % solving JC (4.24) for a (junctions displacements), (checked 110414)
        % a=sum(Rj*Kj*Rj^T)\(Rm*fm)
        a(from).a1 = -(aL12sum.*aL23sum.*aL34sum.*a(from).aR4 - aL12sum.*aL23sum.*aL44sum.*a(from).aR3 - aL12sum.*aL24sum.*aL33sum.*a(from).aR4 + aL12sum.*aL24sum.*aL43sum.*a(from).aR3 + aL12sum.*aL33sum.*aL44sum.*a(from).aR2 - aL12sum.*aL34sum.*aL43sum.*a(from).aR2 - aL13sum.*aL22sum.*aL34sum.*a(from).aR4 + aL13sum.*aL22sum.*aL44sum.*a(from).aR3 + aL13sum.*aL24sum.*aL32sum.*a(from).aR4 - aL13sum.*aL24sum.*aL42sum.*a(from).aR3 - aL13sum.*aL32sum.*aL44sum.*a(from).aR2 + aL13sum.*aL34sum.*aL42sum.*a(from).aR2 + aL14sum.*aL22sum.*aL33sum.*a(from).aR4 - aL14sum.*aL22sum.*aL43sum.*a(from).aR3 - aL14sum.*aL23sum.*aL32sum.*a(from).aR4 + aL14sum.*aL23sum.*aL42sum.*a(from).aR3 + aL14sum.*aL32sum.*aL43sum.*a(from).aR2 - aL14sum.*aL33sum.*aL42sum.*a(from).aR2 - aL22sum.*aL33sum.*aL44sum.*a(from).aR1 + aL22sum.*aL34sum.*aL43sum.*a(from).aR1 + aL23sum.*aL32sum.*aL44sum.*a(from).aR1 - aL23sum.*aL34sum.*aL42sum.*a(from).aR1 - aL24sum.*aL32sum.*aL43sum.*a(from).aR1 + aL24sum.*aL33sum.*aL42sum.*a(from).aR1)./(aL11sum.*aL22sum.*aL33sum.*aL44sum - aL11sum.*aL22sum.*aL34sum.*aL43sum - aL11sum.*aL23sum.*aL32sum.*aL44sum + aL11sum.*aL23sum.*aL34sum.*aL42sum + aL11sum.*aL24sum.*aL32sum.*aL43sum - aL11sum.*aL24sum.*aL33sum.*aL42sum - aL12sum.*aL21sum.*aL33sum.*aL44sum + aL12sum.*aL21sum.*aL34sum.*aL43sum + aL12sum.*aL23sum.*aL31sum.*aL44sum - aL12sum.*aL23sum.*aL34sum.*aL41sum - aL12sum.*aL24sum.*aL31sum.*aL43sum + aL12sum.*aL24sum.*aL33sum.*aL41sum + aL13sum.*aL21sum.*aL32sum.*aL44sum - aL13sum.*aL21sum.*aL34sum.*aL42sum - aL13sum.*aL22sum.*aL31sum.*aL44sum + aL13sum.*aL22sum.*aL34sum.*aL41sum + aL13sum.*aL24sum.*aL31sum.*aL42sum - aL13sum.*aL24sum.*aL32sum.*aL41sum - aL14sum.*aL21sum.*aL32sum.*aL43sum + aL14sum.*aL21sum.*aL33sum.*aL42sum + aL14sum.*aL22sum.*aL31sum.*aL43sum - aL14sum.*aL22sum.*aL33sum.*aL41sum - aL14sum.*aL23sum.*aL31sum.*aL42sum + aL14sum.*aL23sum.*aL32sum.*aL41sum);
        a(from).a2 = (aL11sum.*aL23sum.*aL34sum.*a(from).aR4 - aL11sum.*aL23sum.*aL44sum.*a(from).aR3 - aL11sum.*aL24sum.*aL33sum.*a(from).aR4 + aL11sum.*aL24sum.*aL43sum.*a(from).aR3 + aL11sum.*aL33sum.*aL44sum.*a(from).aR2 - aL11sum.*aL34sum.*aL43sum.*a(from).aR2 - aL13sum.*aL21sum.*aL34sum.*a(from).aR4 + aL13sum.*aL21sum.*aL44sum.*a(from).aR3 + aL13sum.*aL24sum.*aL31sum.*a(from).aR4 - aL13sum.*aL24sum.*aL41sum.*a(from).aR3 - aL13sum.*aL31sum.*aL44sum.*a(from).aR2 + aL13sum.*aL34sum.*aL41sum.*a(from).aR2 + aL14sum.*aL21sum.*aL33sum.*a(from).aR4 - aL14sum.*aL21sum.*aL43sum.*a(from).aR3 - aL14sum.*aL23sum.*aL31sum.*a(from).aR4 + aL14sum.*aL23sum.*aL41sum.*a(from).aR3 + aL14sum.*aL31sum.*aL43sum.*a(from).aR2 - aL14sum.*aL33sum.*aL41sum.*a(from).aR2 - aL21sum.*aL33sum.*aL44sum.*a(from).aR1 + aL21sum.*aL34sum.*aL43sum.*a(from).aR1 + aL23sum.*aL31sum.*aL44sum.*a(from).aR1 - aL23sum.*aL34sum.*aL41sum.*a(from).aR1 - aL24sum.*aL31sum.*aL43sum.*a(from).aR1 + aL24sum.*aL33sum.*aL41sum.*a(from).aR1)./(aL11sum.*aL22sum.*aL33sum.*aL44sum - aL11sum.*aL22sum.*aL34sum.*aL43sum - aL11sum.*aL23sum.*aL32sum.*aL44sum + aL11sum.*aL23sum.*aL34sum.*aL42sum + aL11sum.*aL24sum.*aL32sum.*aL43sum - aL11sum.*aL24sum.*aL33sum.*aL42sum - aL12sum.*aL21sum.*aL33sum.*aL44sum + aL12sum.*aL21sum.*aL34sum.*aL43sum + aL12sum.*aL23sum.*aL31sum.*aL44sum - aL12sum.*aL23sum.*aL34sum.*aL41sum - aL12sum.*aL24sum.*aL31sum.*aL43sum + aL12sum.*aL24sum.*aL33sum.*aL41sum + aL13sum.*aL21sum.*aL32sum.*aL44sum - aL13sum.*aL21sum.*aL34sum.*aL42sum - aL13sum.*aL22sum.*aL31sum.*aL44sum + aL13sum.*aL22sum.*aL34sum.*aL41sum + aL13sum.*aL24sum.*aL31sum.*aL42sum - aL13sum.*aL24sum.*aL32sum.*aL41sum - aL14sum.*aL21sum.*aL32sum.*aL43sum + aL14sum.*aL21sum.*aL33sum.*aL42sum + aL14sum.*aL22sum.*aL31sum.*aL43sum - aL14sum.*aL22sum.*aL33sum.*aL41sum - aL14sum.*aL23sum.*aL31sum.*aL42sum + aL14sum.*aL23sum.*aL32sum.*aL41sum);
        a(from).a3 = -(aL11sum.*aL22sum.*aL34sum.*a(from).aR4 - aL11sum.*aL22sum.*aL44sum.*a(from).aR3 - aL11sum.*aL24sum.*aL32sum.*a(from).aR4 + aL11sum.*aL24sum.*aL42sum.*a(from).aR3 + aL11sum.*aL32sum.*aL44sum.*a(from).aR2 - aL11sum.*aL34sum.*aL42sum.*a(from).aR2 - aL12sum.*aL21sum.*aL34sum.*a(from).aR4 + aL12sum.*aL21sum.*aL44sum.*a(from).aR3 + aL12sum.*aL24sum.*aL31sum.*a(from).aR4 - aL12sum.*aL24sum.*aL41sum.*a(from).aR3 - aL12sum.*aL31sum.*aL44sum.*a(from).aR2 + aL12sum.*aL34sum.*aL41sum.*a(from).aR2 + aL14sum.*aL21sum.*aL32sum.*a(from).aR4 - aL14sum.*aL21sum.*aL42sum.*a(from).aR3 - aL14sum.*aL22sum.*aL31sum.*a(from).aR4 + aL14sum.*aL22sum.*aL41sum.*a(from).aR3 + aL14sum.*aL31sum.*aL42sum.*a(from).aR2 - aL14sum.*aL32sum.*aL41sum.*a(from).aR2 - aL21sum.*aL32sum.*aL44sum.*a(from).aR1 + aL21sum.*aL34sum.*aL42sum.*a(from).aR1 + aL22sum.*aL31sum.*aL44sum.*a(from).aR1 - aL22sum.*aL34sum.*aL41sum.*a(from).aR1 - aL24sum.*aL31sum.*aL42sum.*a(from).aR1 + aL24sum.*aL32sum.*aL41sum.*a(from).aR1)./(aL11sum.*aL22sum.*aL33sum.*aL44sum - aL11sum.*aL22sum.*aL34sum.*aL43sum - aL11sum.*aL23sum.*aL32sum.*aL44sum + aL11sum.*aL23sum.*aL34sum.*aL42sum + aL11sum.*aL24sum.*aL32sum.*aL43sum - aL11sum.*aL24sum.*aL33sum.*aL42sum - aL12sum.*aL21sum.*aL33sum.*aL44sum + aL12sum.*aL21sum.*aL34sum.*aL43sum + aL12sum.*aL23sum.*aL31sum.*aL44sum - aL12sum.*aL23sum.*aL34sum.*aL41sum - aL12sum.*aL24sum.*aL31sum.*aL43sum + aL12sum.*aL24sum.*aL33sum.*aL41sum + aL13sum.*aL21sum.*aL32sum.*aL44sum - aL13sum.*aL21sum.*aL34sum.*aL42sum - aL13sum.*aL22sum.*aL31sum.*aL44sum + aL13sum.*aL22sum.*aL34sum.*aL41sum + aL13sum.*aL24sum.*aL31sum.*aL42sum - aL13sum.*aL24sum.*aL32sum.*aL41sum - aL14sum.*aL21sum.*aL32sum.*aL43sum + aL14sum.*aL21sum.*aL33sum.*aL42sum + aL14sum.*aL22sum.*aL31sum.*aL43sum - aL14sum.*aL22sum.*aL33sum.*aL41sum - aL14sum.*aL23sum.*aL31sum.*aL42sum + aL14sum.*aL23sum.*aL32sum.*aL41sum);
        a(from).a4 = (aL11sum.*aL22sum.*aL33sum.*a(from).aR4 - aL11sum.*aL22sum.*aL43sum.*a(from).aR3 - aL11sum.*aL23sum.*aL32sum.*a(from).aR4 + aL11sum.*aL23sum.*aL42sum.*a(from).aR3 + aL11sum.*aL32sum.*aL43sum.*a(from).aR2 - aL11sum.*aL33sum.*aL42sum.*a(from).aR2 - aL12sum.*aL21sum.*aL33sum.*a(from).aR4 + aL12sum.*aL21sum.*aL43sum.*a(from).aR3 + aL12sum.*aL23sum.*aL31sum.*a(from).aR4 - aL12sum.*aL23sum.*aL41sum.*a(from).aR3 - aL12sum.*aL31sum.*aL43sum.*a(from).aR2 + aL12sum.*aL33sum.*aL41sum.*a(from).aR2 + aL13sum.*aL21sum.*aL32sum.*a(from).aR4 - aL13sum.*aL21sum.*aL42sum.*a(from).aR3 - aL13sum.*aL22sum.*aL31sum.*a(from).aR4 + aL13sum.*aL22sum.*aL41sum.*a(from).aR3 + aL13sum.*aL31sum.*aL42sum.*a(from).aR2 - aL13sum.*aL32sum.*aL41sum.*a(from).aR2 - aL21sum.*aL32sum.*aL43sum.*a(from).aR1 + aL21sum.*aL33sum.*aL42sum.*a(from).aR1 + aL22sum.*aL31sum.*aL43sum.*a(from).aR1 - aL22sum.*aL33sum.*aL41sum.*a(from).aR1 - aL23sum.*aL31sum.*aL42sum.*a(from).aR1 + aL23sum.*aL32sum.*aL41sum.*a(from).aR1)./(aL11sum.*aL22sum.*aL33sum.*aL44sum - aL11sum.*aL22sum.*aL34sum.*aL43sum - aL11sum.*aL23sum.*aL32sum.*aL44sum + aL11sum.*aL23sum.*aL34sum.*aL42sum + aL11sum.*aL24sum.*aL32sum.*aL43sum - aL11sum.*aL24sum.*aL33sum.*aL42sum - aL12sum.*aL21sum.*aL33sum.*aL44sum + aL12sum.*aL21sum.*aL34sum.*aL43sum + aL12sum.*aL23sum.*aL31sum.*aL44sum - aL12sum.*aL23sum.*aL34sum.*aL41sum - aL12sum.*aL24sum.*aL31sum.*aL43sum + aL12sum.*aL24sum.*aL33sum.*aL41sum + aL13sum.*aL21sum.*aL32sum.*aL44sum - aL13sum.*aL21sum.*aL34sum.*aL42sum - aL13sum.*aL22sum.*aL31sum.*aL44sum + aL13sum.*aL22sum.*aL34sum.*aL41sum + aL13sum.*aL24sum.*aL31sum.*aL42sum - aL13sum.*aL24sum.*aL32sum.*aL41sum - aL14sum.*aL21sum.*aL32sum.*aL43sum + aL14sum.*aL21sum.*aL33sum.*aL42sum + aL14sum.*aL22sum.*aL31sum.*aL43sum - aL14sum.*aL22sum.*aL33sum.*aL41sum - aL14sum.*aL23sum.*aL31sum.*aL42sum + aL14sum.*aL23sum.*aL32sum.*aL41sum);
        % calculating the edge displacements of the plates from the beam displacement.
        for subs = 1:numss
            % JC (4.20)  bj=Rj^T*a
            b(from,subs).b1 = a(from).a1;
            b(from,subs).b2 = a(from).a2.*R(subs).R22 + a(from).a3.*R(subs).R32 + a(from).a4.*R(subs).R42;
            b(from,subs).b3 = a(from).a2.*R(subs).R23 + a(from).a3.*R(subs).R33 + a(from).a4.*R(subs).R43;
            b(from,subs).b4 = a(from).a4;
            % Calculate the wave amplitudes in each plate from the edge displacements of that plate
            % JC (4.39) alfa_j=Kaj\bj
            dis(from,subs).dis1 = (K(from,subs).Ka22.*b(from,subs).b1 - K(from,subs).Ka12.*b(from,subs).b2)./(K(from,subs).Ka11.*K(from,subs).Ka22 - K(from,subs).Ka12.*K(from,subs).Ka21);
            dis(from,subs).dis2 = -(K(from,subs).Ka21.*b(from,subs).b1 - K(from,subs).Ka11.*b(from,subs).b2)./(K(from,subs).Ka11.*K(from,subs).Ka22 - K(from,subs).Ka12.*K(from,subs).Ka21);
            dis(from,subs).dis3 = (b(from,subs).b4 - K(from,subs).Kb22.*b(from,subs).b3)./(K(from,subs).Kb21 - K(from,subs).Kb22);
            dis(from,subs).dis4 = -(b(from,subs).b4 - K(from,subs).Kb21.*b(from,subs).b3)./(K(from,subs).Kb21 - K(from,subs).Kb22);
            if ~(from == subs)
                % Calculate the power carried by the transmitted waves
                % JC (4.40-4.42)
                P(from,subs).Pb1 = ((rho(subs)*(ones(numang,1)*w).^3.*abs(dis(from,subs).dis3).^2)./(ones(numang,1)*k(subs).b)).*sind(real(angtr(from,subs).waveB));
                P(from,subs).Pb2 = ((rho(subs)*(ones(numang,1)*w).^3.*abs(dis(from,subs).dis4).^2)./(ones(numang,1)*k(subs).b)).*sind(real(angtr(from,subs).waveB));
                P(from,subs).Pl = 0.5*rho(subs)*(ones(numang,1)*w).^3.*(ones(numang,1)*k(subs).l).*abs(dis(from,subs).dis1).^2.*sind(real(angtr(from,subs).waveL));
                P(from,subs).Ps = 0.5*rho(subs)*(ones(numang,1)*w).^3.*(ones(numang,1)*k(subs).s).*abs(dis(from,subs).dis2).^2.*sind(real(angtr(from,subs).waveS));
            else
                % Calculate the wave amplitudes of the incident wave in the subsystem carrying the 
                %   incident wave from the edge displacements of that plate
                % JC (4.39) alfa'_j=Kaj\bj'
                dis(from,subs).dismp1 = (K(from,subs).Ka22.*bp(from).p_1 - K(from,subs).Ka12.*bp(from).p_2)./(K(from,subs).Ka11.*K(from,subs).Ka22 - K(from,subs).Ka12.*K(from,subs).Ka21);
                dis(from,subs).dismp2 = -(K(from,subs).Ka21.*bp(from).p_1 - K(from,subs).Ka11.*bp(from).p_2)./(K(from,subs).Ka11.*K(from,subs).Ka22 - K(from,subs).Ka12.*K(from,subs).Ka21);
                dis(from,subs).dismp3 = (bp(from).p_4 - K(from,subs).Kb22.*bp(from).p_3)./(K(from,subs).Kb21 - K(from,subs).Kb22);
                dis(from,subs).dismp4 = -(bp(from).p_4 - K(from,subs).Kb21.*bp(from).p_3)./(K(from,subs).Kb21 - K(from,subs).Kb22);
                % Calculate the power carried by the reflected wave in the subsystem of the incident wave
                % JC (4.40-4.42) with the wave amplitude alfa_j-alfa'_j (amplitude of incident wave subtracted
                %       from the total wave amplitude in the plate gives the amplitude of the reflected wave)
                P(from,subs).Pb1 = ((rho(subs)*(ones(numang,1)*w).^3.*abs((dis(from,subs).dis3-dis(from,subs).dismp3)).^2)./(ones(numang,1)*k(subs).b)).*sind(real(angtr(from,subs).waveB));
                P(from,subs).Pb2 = ((rho(subs)*(ones(numang,1)*w).^3.*abs((dis(from,subs).dis4-dis(from,subs).dismp4)).^2)./(ones(numang,1)*k(subs).b)).*sind(real(angtr(from,subs).waveB));
                P(from,subs).Pl = 0.5*rho(subs)*(ones(numang,1)*w).^3.*(ones(numang,1)*k(subs).l).*abs((dis(from,subs).dis1-dis(from,subs).dismp1)).^2.*sind(real(angtr(from,subs).waveL));
                P(from,subs).Ps = 0.5*rho(subs)*(ones(numang,1)*w).^3.*(ones(numang,1)*k(subs).s).*abs((dis(from,subs).dis2-dis(from,subs).dismp2)).^2.*sind(real(angtr(from,subs).waveS));
            end
        end
        % calculate the input power carried by the incident wave.
        switch wavetype
            case 1, Pin(from).Pin = (rho(from)*(ones(numang,1)*w).^3.*alfa(from).b.^2./(ones(numang,1)*k(from).b)).*(sind(ang(from).wave.')*ones(1,numf));
            case 2, Pin(from).Pin = 0.5*rho(from)*(ones(numang,1)*w).^3.*(ones(numang,1)*k(from).l).*alfa(from).l.^2.*(sind(ang(from).wave.')*ones(1,numf));
            case 3, Pin(from).Pin = 0.5*rho(from)*(ones(numang,1)*w).^3.*(ones(numang,1)*k(from).s).*alfa(from).s.^2.*(sind(ang(from).wave.')*ones(1,numf));
        end
        % the transmission coefficient is a ratio between the transmitted
        % power and the incident power.
        for subs = 1:numss
            % taking the intrgral over all calculated angles. trapezoidal rule
            %   excluding angle 0 degrees for which Pin=0 and tau_ij=0            
            tau(from,subs,wavetype).b = 0.5*sum((P(from,subs).Pb2(2:end-1,:)./Pin(from).Pin(2:end-1,:)).*((sind(ang(from).wave(2:end-1)).*ang_dif(2:end))'*ones(1,length(f))),1);
            tau(from,subs,wavetype).b = tau(from,subs,wavetype).b + 0.5*sum((P(from,subs).Pb2(2:end,:)./Pin(from).Pin(2:end,:)).*((sind(ang(from).wave(2:end)).*ang_dif(1:end))'*ones(1,length(f))),1);
            tau(from,subs,wavetype).l = 0.5*sum((P(from,subs).Pl(2:end-1,:)./Pin(from).Pin(2:end-1,:)).*((sind(ang(from).wave(2:end-1)).*ang_dif(2:end))'*ones(1,length(f))),1);
            tau(from,subs,wavetype).l = tau(from,subs,wavetype).l + 0.5*sum((P(from,subs).Pl(2:end,:)./Pin(from).Pin(2:end,:)).*((sind(ang(from).wave(2:end)).*ang_dif(1:end))'*ones(1,length(f))),1);
            tau(from,subs,wavetype).s = 0.5*sum((P(from,subs).Ps(2:end-1,:)./Pin(from).Pin(2:end-1,:)).*((sind(ang(from).wave(2:end-1)).*ang_dif(2:end))'*ones(1,length(f))),1);
            tau(from,subs,wavetype).s = tau(from,subs,wavetype).s + 0.5*sum((P(from,subs).Ps(2:end,:)./Pin(from).Pin(2:end,:)).*((sind(ang(from).wave(2:end)).*ang_dif(1:end))'*ones(1,length(f))),1);
        end
    end
end
% return the transmission coefficients.
return_data = tau;
