function return_data = calc_k(subsystem_values,f,type)
% calc_k(subsystem_values,f,type) returns a structure array containing
% the wave numbers for the given subsystems and the given frequencies.
%
% [Usage]:
%   k = calc_k(subsystem_values,f,type)
%
% [Inputs]:
%   subsystem_values (structure) - fields (for plates):
%                                       rho - density
%                                       thickness - plate thickness
%                                       D - bending stiffness
%                                       poi - Poisson's ratio
%                                       E - Elasticity modulus, Youngs
%                                - fields (for cavities):
%                                       c - wave speed in the fluid
%
%   f (double array)             - Frequency vector
%   type (string)                - subsystem type, can either be: 
%                                  'plate' or 'cavity'
%
% [Output]:
%   k (structure) - fields: b - Bending wave number
%                           l - Longitudinal wave number
%                           s - Shear wave number
%

% load all material properties and keep them persistent within the function
% to prevent unnecessary load times.
persistent solids
if (isempty(solids))
    load physicalproperties solids
end
w = 2*pi*f; % angular frequency
% calculate the wave numbers for the given subsystems
switch type
    case 'plate'
        % k_B=(rho*t*w^2/D)^(1/4)
        k.b = (subsystem_values.rho*subsystem_values.thickness*w.^2/subsystem_values.D).^(1/4);
        % k_L=sqrt(rho*t*w^2*(1-mu^2)/E*t)
        k.l = sqrt(subsystem_values.rho*subsystem_values.thickness*w.^2*(1-subsystem_values.poi^2)/(subsystem_values.E*subsystem_values.thickness));
        % k_s=sqrt(2*rho*t*w^2*(1+mu)/E*t)
        k.s = sqrt(2*subsystem_values.rho*subsystem_values.thickness*w.^2*(1+subsystem_values.poi)/(subsystem_values.E*subsystem_values.thickness));
    case 'cavity'
        k = 2*pi*f./subsystem_values.c;
end
% return wave number structure array
return_data = k;