function return_data = energy_calc_v0p0(data)
% energy_calc(data) calculates and returns the energies in each subsystem
% in the SEA-system.
%
%  [Usage]: data = energy_calc_v0p0(data)
%           
%  [Inputs]: 
%   data (structure array)
%       fields:
%           f (double array) - frequency vector
%           numcon - number of connections in the subssystem.
%           numss - number of subsystems in the SEA-system.
%           connections(ii) (structure) - structure with all connection
%                                         dependant properties. with index
%                                         (ii) representing each connection
%             fields:
%               subsystems (double array) - vector containing all
%                                           subsystems in that connection.
%               clf (structure) - clf(subsystem1,subsystem2,wavetype)
%                                 with fields: 'b', 'l' and 's'.
%                                 subsystem1 and subsystem2 represents the
%                                 clf from subsystem1 to subsystem2 with a
%                                 incident energy of a certain energy
%                                 described by the third input argument.
%                                 wavetype=1 represents bending energy,
%                                 wavetype=2 longidutinal and wavetype=3
%                                 shear energy. The fields represent the
%                                 energy in subsystem2, 'b' for bending
%                                 energy, 'l' for longitudinal energy and
%                                 's' for shear energy.
%
%           subsystems(ii) (structure) - structure with all subsystem
%                                        dependant properties. with index
%                                        (ii) representing each subsystem.
%             fields:
%               values (structure)
%                 fields:
%                   damping (structure)
%                     fields:
%                       b (double array) - total loss factor for bending
%                                          waves.
%                       l (double array) - total loss factor for
%                                          longitudinal waves.
%                       s (double array) - total loss factor for shear
%                                          waves.
%                   input_power (structure)
%                     fields:
%                       power (structure)
%                         fields:
%                           b (double array) - bending input power.
%                           l (double array) - longitudinal input power.
%                           s (double array) - shear input power.
%
% [Output]:
%   data (structure) - same structure as the data structure from the input
%                      argument but with added fields:
%       subsystems(ii).Energy
%                       with fields:
%                           b - bending energy in subsystem ii
%                           l - longitudinal energy in subsystem ii
%                           s - shear energy in subsystem ii
%

% Updates by Patrik Andersson
% 110405 PA Adding some comments
% 110406 PA New name: energy_calc_v0p0.m
% 110406 PA Trying to resolve the issue that losses due to internal dissipation of a subsystem 
%           is included for each connection it is connected to but it should only be included once
% 110428 PA Corrected the alogorithm so that losses due to internal dissipation loss factor of a 
%           subsystem is added to the diagonal of the matrix only once (before it was added each 
%           time the subsystem showed up in a new connection). The updated algorithm does one use
%           the field 'mat().losses' which has been removed also in the memory allocation.

f = data.f;
numf = length(f);
numss = data.numss;
for ii = 1:numss
    if length(data.subsystems(ii).values.damping.b)==1
        data.subsystems(ii).values.damping.b = data.subsystems(ii).values.damping.b.*ones(1,length(f));
    end
    if length(data.subsystems(ii).values.damping.l)==1
        data.subsystems(ii).values.damping.l = data.subsystems(ii).values.damping.l.*ones(1,length(f));
    end
    if length(data.subsystems(ii).values.damping.s)==1
        data.subsystems(ii).values.damping.s = data.subsystems(ii).values.damping.s.*ones(1,length(f));
    end
    if length(data.subsystems(ii).values.input_power.power.b)==1
        data.subsystems(ii).values.input_power.power.b = data.subsystems(ii).values.input_power.power.b.*ones(1,length(f));
    end
    if length(data.subsystems(ii).values.input_power.power.l)==1
        data.subsystems(ii).values.input_power.power.l = data.subsystems(ii).values.input_power.power.l.*ones(1,length(f));
    end
    if length(data.subsystems(ii).values.input_power.power.s)==1
        data.subsystems(ii).values.input_power.power.s = data.subsystems(ii).values.input_power.power.s.*ones(1,length(f));
    end
end
% predifing variables to prevent reallocating memory in for loops.
mat(1:data.numcon) = struct('gains',zeros(numss*3,numss*3,numf));
mat_lg = zeros(numss*3,numss*3,numf);
P(1:numss*3,1:numf) = 0;
% create "clf-matrix" for the energy calculation.
for con = 1:data.numcon % For each connection
    for ff = 1:numf % For each frequency
        % Gains to the subsystems (off-diagonals in the matrix)
        for subs1 = data.connections(con).subsystems % For each connected subsystem ("from" subsystems)
            subs1_pos = find(data.connections(con).subsystems == subs1); % Find index of "from" subsystem
            for subs2 = data.connections(con).subsystems % For each connected subsystem ("to" subsystems)
                subs2_pos = find(data.connections(con).subsystems == subs2); % Find index of "to" subsystem
                mat(con).gains(subs1*3-2,subs2*3-1,ff) = -data.connections(con).clf(subs2_pos,subs1_pos,2).b(ff); % l to b
                mat(con).gains(subs1*3-2,subs2*3,ff) = -data.connections(con).clf(subs2_pos,subs1_pos,3).b(ff); % s to b
                mat(con).gains(subs1*3-1,subs2*3-2,ff) = -data.connections(con).clf(subs2_pos,subs1_pos,1).l(ff); % b to l
                mat(con).gains(subs1*3-1,subs2*3,ff) = -data.connections(con).clf(subs2_pos,subs1_pos,3).l(ff); % s to l
                mat(con).gains(subs1*3,subs2*3-2,ff) = -data.connections(con).clf(subs2_pos,subs1_pos,1).s(ff); % b to s
                mat(con).gains(subs1*3,subs2*3-1,ff) = -data.connections(con).clf(subs2_pos,subs1_pos,2).s(ff); % l to s
                if ~(subs1==subs2) % If different subsystems also b to b, l to l, and s to s connections, otherwise not
                    mat(con).gains(subs1*3-2,subs2*3-2,ff) = -data.connections(con).clf(subs2_pos,subs1_pos,1).b(ff); % b to b
                    mat(con).gains(subs1*3-1,subs2*3-1,ff) = -data.connections(con).clf(subs2_pos,subs1_pos,2).l(ff); % l to l
                    mat(con).gains(subs1*3,subs2*3,ff) = -data.connections(con).clf(subs2_pos,subs1_pos,3).s(ff); % s to s
                end
            end
        end
    end
    mat_lg = mat_lg + mat(con).gains;
end
for ff = 1:numf % For each frequency
    for subs1 = 1:numss
        % Losses to other subsystems
        for con=1:data.numcon
            mat_lg(subs1*3-2,subs1*3-2,ff) = mat_lg(subs1*3-2,subs1*3-2,ff)+sum(abs(mat(con).gains(:,subs1*3-2,ff)));
            mat_lg(subs1*3-1,subs1*3-1,ff) = mat_lg(subs1*3-1,subs1*3-1,ff)+sum(abs(mat(con).gains(:,subs1*3-1,ff)));
            mat_lg(subs1*3,subs1*3,ff) = mat_lg(subs1*3,subs1*3,ff)+sum(abs(mat(con).gains(:,subs1*3,ff)));
        end
        % Internal dissipation
        mat_lg(subs1*3-2,subs1*3-2,ff) = mat_lg(subs1*3-2,subs1*3-2,ff)+data.subsystems(subs1).values.damping.b(ff);
        mat_lg(subs1*3-1,subs1*3-1,ff) = mat_lg(subs1*3-1,subs1*3-1,ff)+data.subsystems(subs1).values.damping.l(ff);
        mat_lg(subs1*3,subs1*3,ff) = mat_lg(subs1*3,subs1*3,ff)+data.subsystems(subs1).values.damping.s(ff);
    end
end

% create the power vector needed to calculate the energy.
for ff = 1:numf
    mat_lg(:,:,ff)=mat_lg(:,:,ff).*(2*pi*f(ff));
    for subs = 1:numss
        P(subs*3-2,ff) = data.subsystems(subs).values.input_power.power.b(ff);
        P(subs*3-1,ff) = data.subsystems(subs).values.input_power.power.l(ff);
        P(subs*3,ff) = data.subsystems(subs).values.input_power.power.s(ff);
    end
    % calculate the energy
    E(ff).Etot = mat_lg(:,:,ff)\P(:,ff);
    % assign the energy to the according subsystem in the data structure.
    for subs = 1:numss
        data.subsystems(subs).Energy.b(ff) = E(ff).Etot(subs*3-2);
        data.subsystems(subs).Energy.l(ff) = E(ff).Etot(subs*3-1);
        data.subsystems(subs).Energy.s(ff) = E(ff).Etot(subs*3);
    end
end
% return the data structure containing the energies in each subsystem.
return_data = data;
