function return_data = mden_calc(f,subsystem_values,type)
% mden_calc(f,subsystem_values,type) returns the modal density of plates
% and cavities. Modes of bending, longitudinal and shear waves are treated.
%
% [Usage]:
%   mden = mden_calc(f,subsystem_values,type)
%
% [Inputs]:
%   f (double array) - frequency vector.
%   typ (string array) - type of the subsystem considered.
%                        Ex. 'plate' or 'cavity'.
%   subsystem_values (structure array)
%                   - fields (for plates):
%                       k (structure array) with fields:
%                           b - bending wavenumbers
%                           l - longitudinal wave numbers
%                           s - shear wave numbers
%                       c (structure array) with fields:
%                           b - bending wave speeds
%                           l - longitudinal wave speeds
%                           s - shear wave speeds
%                       dim - double array with two values:
%                             first = plate length
%                             second = plate width
%                   - fields (for cavities):
%                       rho - fluid density
%                       k - longitudinal wave numbers in the fluid
%                       c - wave speed in the fluid
%                       volume - cavity volume [m^3]
%                       dim - double array with three values:
%                             first = cavity length
%                             second = cavity height
%                             third = cavity depth
%
% [Output]:
%   mden (structure array) - 
%                   - fields (for plates):
%                       b - modal density, bending waves
%                       l - modal density, longitudinal waves
%                       s - modal density, shear waves
%                   - fields (for plates):
%                       l - modal density, longitudinal waves
%

% Updates by PA
% 110515 Corrected the calculation of f1 (2D / 3D cavity limit)
%           it now says f1=c0/(2*l), before it said f1=rho0/(2*l)
% 110515 Corrected the assignment of the modal density for a cavity
%           it now says mden.l(f<f1)=..., before it said mden.b(f<f1)=...


% calculate the modal density of the given subsystem
if strcmp(type,'plate')
    area = subsystem_values.dim(1)*subsystem_values.dim(2);
    mden.b = (area*subsystem_values.k.b)./(4*pi*subsystem_values.c.b);
    mden.l = (area*subsystem_values.k.l)./(2*pi*subsystem_values.c.l);
    mden.s = (area*subsystem_values.k.s)./(2*pi*subsystem_values.c.s);
elseif strcmp(type,'cavity')
    f1 = subsystem_values.c/(2*subsystem_values.dim(3));
    mden.l(f<f1) = subsystem_values.dim(1)*subsystem_values.dim(2)*subsystem_values.k(f<f1)./(2*pi*subsystem_values.c);
    b = subsystem_values.dim(1);
    h = subsystem_values.dim(2);
    d = subsystem_values.dim(3);
    Sp = 2*(b*d+b*h+d*h);
    Lp = 4*(b+d+h);
    mden.l(f>=f1) = (4*pi*f(f>=f1).^2*subsystem_values.volume./subsystem_values.c^3+pi*f(f>=f1)*Sp./(2*subsystem_values.c^2)+Lp/(8*subsystem_values.c))/(2*pi);
end
% return the modal density
return_data = mden;


