function return_values = set_subsystem_values(type,subsystem_values)
% set_subsystem_values(type,subsystem_values) add/updates fields to the
% given structure array subsystem_values with data corresponding to the
% given type and material of the subsystem.
% 
% [Usage]:
%   subsystem_values = set_subsystem_values(type,subsystem_values)
%
% [Inputs]:
%   type (string array) - type of the subsystem, ex. 'plate', 'cavity' or
%                         'semiinfinitefluid'.
%   subsystem_values (structure array)
%                   - fields (for plates):
%                   	material - vector of the corresponing material
%                                  numbers for the subsystems
%                                  (corresponding to the material numbers
%                                  in the data file physicalproperties.mat)
%                   	thickness - the plate thickness [m]
%                   - fields (for cavities and semiinfinitefluids):
%                   	material - vector of the corresponing material
%                                  numbers for the subsystems
%                                  (corresponding to the material numbers
%                                  in the data file physicalproperties.mat)
%
% [Output]:
%   subsystem_values (structure array)
%                   - added/updated fields (for plates):
%                       material_type - string array containing the name of
%                                       the material.
%                       E - value of the youngs elasticity modulus of the
%                           plate.
%                       poi - Poissons ratio of the plate material.
%                       rho - density of the plate material.
%                       D - bending stiffness of the plate.
%                   - added/updated fields
%                     (for cavities and semiinfinitefluids):
%                       fluid_type - string array containing the name of
%                                    the fluid.
%                       c - wave speed in the fluid.
%                       rho - density of the fluid.
%                       vis - viscosity of the fluid.

% load all material properties and keep them persistent within the function
% to prevent unnecessary load times.
persistent solids fluids
if (isempty(solids) || isempty(fluids))
    load physicalproperties solids fluids
end
% save corresponing material parameters for the certain material type
% to a structure array
if strcmp(type,'plate')
    subsystem_values.material_type = solids.name{subsystem_values.material};
    subsystem_values.E = solids.Ey(subsystem_values.material);
    subsystem_values.poi = solids.poi(subsystem_values.material);
    subsystem_values.rho = solids.rho(subsystem_values.material);
    subsystem_values.D = (solids.Ey(subsystem_values.material)*subsystem_values.thickness^3)/(12*(1-solids.poi(subsystem_values.material)^2));
elseif strcmp(type,'cavity')
    subsystem_values.fluid_type = fluids.name{subsystem_values.material};
    subsystem_values.c = fluids.c(subsystem_values.material);
    subsystem_values.rho = fluids.rho(subsystem_values.material);
    subsystem_values.vis = fluids.vis(subsystem_values.material);
elseif strcmp(type,'semiinfinitefluid')
    subsystem_values.fluid_type = fluids.name{subsystem_values.material};
    subsystem_values.c = fluids.c(subsystem_values.material);
    subsystem_values.rho = fluids.rho(subsystem_values.material);
    subsystem_values.vis = fluids.vis(subsystem_values.material);
end
% return the subsystem parameters
return_values = subsystem_values;