clear all
load('../results/results_double_wall.mat')


%******************************************************************************
% Energy in subsystems
%******************************************************************************
% data.subsystems(n).Energy.b : Bending wave energy of subsystem n
% data.subsystems(n).Energy.l : Longitudinal wave energy of subsystem n
% data.subsystems(n).Energy.s : Shear wave energy of subsystem n
figure(1)
semilogx(data.f,10*log10(data.subsystems(1).Energy.l),'bo-')
hold on
semilogx(data.f,10*log10(data.subsystems(2).Energy.b),'go-')
semilogx(data.f,10*log10(data.subsystems(3).Energy.l),'bv-')
semilogx(data.f,10*log10(data.subsystems(4).Energy.b),'gv-')
semilogx(data.f,10*log10(data.subsystems(5).Energy.l),'b*-')
hold off
xlim([10 20000])
set(gca,'FontSize',14)
xlabel('Frequency [Hz]')
ylabel('Energy level [dB re. 1J]')
legend( 'Cavity 1 L','Plate 1 B','Cavity 2 L','Plate 2 B','Cavity 3 L',3)

%******************************************************************************
% Engineering units, sound pressure levels
%******************************************************************************
% rms-pressure squared are given by subsystem energy times rho*c^2 
%   divided by subsystem volume
ii=[1 3 5]; % Indexes of the subsystems defining the cavities
for nn=1:length(ii)
    V=data.subsystems(ii(nn)).values.volume;
    c=data.subsystems(ii(nn)).values.c;
    rho=data.subsystems(ii(nn)).values.rho;
    prms2(nn,:)=data.subsystems(ii(nn)).Energy.l*rho*c^2/V;
end
figure(2)
semilogx(data.f,10*log10(prms2(1,:)),'bo-')
hold on
semilogx(data.f,10*log10(prms2(2,:)),'bv-')
semilogx(data.f,10*log10(prms2(3,:)),'b*-')
hold off
xlim([10 20000])
set(gca,'FontSize',14)
xlabel('Frequency [Hz]')
ylabel('Sound pressure level [dB re. 20 {\mu}Pa]')
legend('Cavity 1','Cavity 2','Cavity 3',3)


%******************************************************************************
% Engineering units, velocity levels
%******************************************************************************
% rms-velocity squared are given by subsystem energy divided by subsystem mass
ii=[2 4]; % Indexes of the subsystems defining the plate
for nn=1:length(ii)
    thickness=data.subsystems(ii(nn)).values.thickness;
    length=data.subsystems(ii(nn)).values.dim(1);
    width=data.subsystems(ii(nn)).values.dim(2);
    volume=length*width*thickness;
    mass(ii(nn))=volume*data.subsystems(ii(nn)).values.rho;
end
figure(3)
semilogx(data.f,10*log10(data.subsystems(2).Energy.b/mass(2)),'go-.')
hold on
semilogx(data.f,10*log10(data.subsystems(4).Energy.b/mass(4)),'gv-')
hold off
xlim([10 20000])
set(gca,'FontSize',14)
xlabel('Frequency [Hz]')
ylabel('Velocity level [dB re. 1 m/s]')
legend('Plate 1 B','Plate 2 B',3)


%******************************************************************************
% Power input to cavity 3
%******************************************************************************
% Coupling loss factors
CLF_15=data.connections(1).clf(1,5,2).l;clf1='Cavity 1 to Cavity 3 (double wall)';
CLF_25=data.connections(1).clf(2,5,1).l;clf2='Plate 1 B to Cavity 3 (not included)';
CLF_35=data.connections(1).clf(3,5,2).l;clf3='Cavity 2 to Cavity 3 (mass law)';
CLF_45=data.connections(1).clf(4,5,1).l;clf4='Plate 2 B to Cavity 3 (resonant)';

E_1=data.subsystems(1).Energy.l; % Longitudinal wave energy in cavity 1
E_2=data.subsystems(2).Energy.b; % Bending wave energy in plate 1
E_3=data.subsystems(3).Energy.l; % Longitudinal wave energy in cavity 2
E_4=data.subsystems(4).Energy.b; % Bending wave energy in plate 2

% P_ij=omega*eta_ij*E_i;
Pin_15=2*pi*data.f.*CLF_15.*E_1;
Pin_25=2*pi*data.f.*CLF_25.*E_2;
Pin_35=2*pi*data.f.*CLF_35.*E_3;
Pin_45=2*pi*data.f.*CLF_45.*E_4;

% Total power input
Ptot=Pin_15+Pin_25+Pin_35+Pin_45;

figure(4)
semilogx(data.f,10*log10(Ptot),'ko-','LineWidth',2)
hold on
semilogx(data.f,10*log10(Pin_15),'bo-')
semilogx(data.f,10*log10(Pin_25),'go-')
semilogx(data.f,10*log10(Pin_35),'bv-')
semilogx(data.f,10*log10(Pin_45),'gv-')
hold off
xlim([10 20000])
set(gca,'FontSize',14)
xlabel('Frequency [Hz]')
ylabel('Power input level [dB re. 1 Watt]')
legend('Total to cavity 3',clf1,clf2,clf3,clf4,3)	

%******************************************************************************
% Coupling loss factors at the Junction
%******************************************************************************
% data.connections(n1).clf(n2,n3,n4).n5 : 
%       n1 : number of the junction
%       n2 : from subsystem n2 at the specific junction
%       n3 : to subsystem n3 at the specific junction
% (n2 and n3 refers to the indexes of the list data.connections(n1).subsystems 
%       where the index of the connected cavities and plate are listed)
%       n4 : wave type in subsystem n2 at the juction
%            n4=1 - bending, n4=2 - long., and n4=3 - shear
%       n5 : wave type in subsystem n3 at the junction
%            n5='b' - bending, n5='l' - long., and n5='s' - shear
CLF_15=data.connections(1).clf(1,5,2).l;clf1='Cavity 1 to Cavity 3 (double wall)';
CLF_25=data.connections(1).clf(2,5,1).l;clf2='Plate 1 B to Cavity 3 (not included)';
CLF_35=data.connections(1).clf(3,5,2).l;clf3='Cavity 2 to Cavity 3 (mass law)';
CLF_45=data.connections(1).clf(4,5,1).l;clf4='Plate 2 B to Cavity 3 (resonant)';

figure(5)
semilogx(data.f,10*log10(CLF_15),'bo-','LineWidth',1)
hold on
semilogx(data.f,10*log10(CLF_25),'go-','LineWidth',1)
semilogx(data.f,10*log10(CLF_35),'bv-','LineWidth',1)
semilogx(data.f,10*log10(CLF_45),'g*-','LineWidth',1)
hold off
xlim([10 20000])
%set(gca,'YScale','log') 
set(gca,'FontSize',14)
xlabel('Frequency [Hz]')
ylabel('Coupling loss factors [dB]')
legend(clf1,clf2,clf3,clf4,3)

%******************************************************************************
% Reduction index
%******************************************************************************
% Using the formula R=L_S-L_R-10*log10(A_R/S)
%   L_S - Sound pressure level in sending room (cavity 1) [dB re 20e-6 Pa]
%   L_R - Sound pressure level in receiving room (cavity 5) [dB re 20e-6 Pa]
%   A_R - Equivalent absorption area in receiving room [m^2 Sabine]
%   S   - Area of the parition [m^2]

% Sound pressure level
pref2=(2e-5)^2; % Reference pressure
Lp_S=10*log10(prms2(1,:)/pref2);
Lp_R=10*log10(prms2(3,:)/pref2);

% Additiona data
S=data.connections(1).values.area; % Area of partition
V_R=data.subsystems(5).values.volume; % Volume of receiving room
eta=data.subsystems(5).values.damping.l; % Loss factor in receiving room
A_R=1/(6*log(10))*data.f.*eta*V_R; % From loss factor to equivalent absorption area
R=Lp_S-Lp_R-10*log10(A_R/S); % Reduction index

m1=data.subsystems(2).values.rho*data.subsystems(2).values.thickness;
m2=data.subsystems(4).values.rho*data.subsystems(4).values.thickness;
R_masslaw=20*log10((m1+m2)*data.f)-47;
R1_masslaw=20*log10(m1*data.f)-47;
R2_masslaw=20*log10(m2*data.f)-47;
R_dw_approx=R1_masslaw+R1_masslaw+20*log10(data.f*data.subsystems(3).values.dim(3))-29;

figure(6)
semilogx(data.f,R,'b','LineWidth',2)
hold on
semilogx(data.f,R_masslaw,'g')
semilogx(data.f,R_dw_approx,'r')
hold off
xlim([10 20000])
set(gca,'FontSize',14)
xlabel('Frequency [Hz]')
ylabel('Reduction index [dB]')
legend('SEAlab including resonant transmission','Masslaw of whole wall','Double wall if plates acts like masses',2)


